/**
  PasswordMaker - Creates and manages passwords
  Copyright (C) 2005 Eric H. Jung and LeahScape, Inc.
  http://passwordmaker.org/
  grimholtz@yahoo.com

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or (at
  your option) any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESSFOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
**/

// References to the UI elements
var passwordMaster, passwordMaster2, passwordGenerated, passwordStorageLB, passwordMaker,
  createTreeFolderBtn, newAccountCmd, usingURL, confirmMPWCB;

var NONE_CLICK_ACCOUNT_SETTINGS="[none - click account settings to set]";

// State
var previousPasswordStorageLBValue, clipboardContents;
const nsIFilePicker = Components.interfaces.nsIFilePicker;

/**
 * Called when the dialog box is opening.
 */
function init() {
  passwordMaker = window.opener.passwordMaker;

  // Get references to the UI elements
  passwordMaster = document.getElementById("passwordMaster");
  passwordMaster2 = document.getElementById("passwordMaster2");
  passwordGenerated = document.getElementById("passwordGenerated");
  passwordStorageLB = document.getElementById("passwordStorageLB");
  createTreeFolderBtn = document.getElementById("createTreeFolderBtn");
  newAccountCmd = document.getElementById("newAccountCmd");
  usingURL = document.getElementById("usingURL");
  confirmMPWCB = document.getElementById("confirmMPWCB");

  initTrees();
  initGlobalSettingsPanel();
  initDomainsPanel();

  var maskGeneratedPassword = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.maskMasterPasswordPredicateRes) == "true";
  passwordGenerated.setAttribute("type", maskGeneratedPassword ? "password" : "");
  refreshMPW(true);
  document.getElementById("mpwConfirmationRow").hidden = !confirmMPWCB.checked;
  passwordMaster.focus();
}

/**
 * Called when user makes UI changes. If the user wants to store the mpw,
 * it is stored with rijndael (AES) encryption.
 */
function _update(quiet) {
  if (passwordMaster.value == "") {
    passwordStorageLB.disabled = true;
    if (passwordStorageLB.value == "store-on-disk") {
      if (!quiet) // shut the fuck up
        passwordMaker.prompts.alert(this, "PasswordMaker", "The encrypted master password has been removed from disk.");
      previousPasswordStorageLBValue = passwordStorageLB.value = "do-not-store";
    }
    hideMPWFieldCB.checked = false;
    PwdMkr_RDFUtils.updateTarget(passwordMaker.globalSettingsSubjectRes, "maskMasterPassword", "false");
  }
  else
    passwordStorageLB.disabled = false;
  PwdMkr_MPW.storeOrClearMasterPassword(passwordStorageLB.value, passwordMaster.value, false);
}

/**
 * Copy _str_ to the clipboard. If _str_ is "", the clipboard
 * is effectively cleared.
 */
function onCopyToClipboard(str) {
  const clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
  clipboardHelper.copyString(str);

  // Auto-clear the clipboard?
  if (passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.autoClearClipboardPredicateRes) == "true") {
    // Yes. Store the data we've copied there so we know whether or not
    // something else has been copied into the clipboard since we put our data there.
    // Notice we're only storing it in this global var *iff* autoClearClipboard is true (more secure that way)
    clipboardContents = str;    
    //Get # of seconds
    var secs = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.autoClearClipboardSecondsPredicateRes);
    setTimeout(conditionallyClearClipboard, secs*1000);
  }
}

/**
 * Clear clipboard but only if
 * it still contains the data we placed there
 */
function conditionallyClearClipboard() {
  try {
    var clip = Components.classes["@mozilla.org/widget/clipboard;1"].getService(Components.interfaces.nsIClipboard);
    if (!clip)
      return;

    var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
    if (!trans)
      return;
    trans.addDataFlavor("text/unicode");
    clip.getData(trans, clip.kGlobalClipboard);
    var str = new Object(), strLength = new Object();
    trans.getTransferData("text/unicode", str, strLength);
    if (str)
      str = str.value.QueryInterface(Components.interfaces.nsISupportsString);
    var clipContents;
    if (str) {
      clipContents = str.data.substring(0,strLength.value / 2);
      if (clipContents == clipboardContents) {
        // Clear the clipboard
        const clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
        clipboardHelper.copyString("");
      }
    }
  }
  catch (e) {
    // There's probably not text/unicode data
    // on the clipboard so let it be.
  }
  finally {
    clipboardContents = null; // Clear internal data member for proactive security
  }
}

function onPasswordStorageLBChanged() {
  if (previousPasswordStorageLBValue == "store-on-disk") {
    if (passwordStorageLB.value != "store-on-disk") {
      passwordMaster.value = passwordMaster2.value = "";
      generatepassword();
      passwordMaker.prompts.alert(this, "PasswordMaker", "The encrypted master password has been removed from disk.");
      hideMPWFieldCB.checked = false; // http://forums.passwordmaker.org/index.php?showtopic=111
    }
  }
  else {
    if (passwordStorageLB.value == "store-on-disk") {
      if (passwordMaker.ask(this, "Although the master password is stored encrypted on disk,\nit is still a security risk to store it there because the\npassword can be decrypted fairly easily.\nAre you sure?")) {
        passwordMaker.prompts.alert(this, "PasswordMaker", "You can remove the master password from disk\nby selecting one of the other storage options or clearing the master password field.");
      }
      else {
        // User canceled
        passwordStorageLB.value = previousPasswordStorageLBValue;
        return;
      }
    }
  }
  if (previousPasswordStorageLBValue == "store-in-memory" && passwordStorageLB.value == "do-not-store") {
    passwordMaster.value = passwordMaster2.value = "";
    generatepassword()
  }
  previousPasswordStorageLBValue = passwordStorageLB.value;
}

function generatepassword() {
  if (passwordMaster.value == "") {
    passwordGenerated.value = passwordMaker.ENTER_MPW;
    return;
  }
  if (confirmMPWCB.checked && passwordMaster.value != passwordMaster2.value) {
    passwordGenerated.value = "passwords do not match";
    return;
  }
  var cells = getTreeCells(accountsTree);
  passwordGenerated.value = passwordMaker.generatepassword(
    cells[4].getAttribute("label"),  // hashAlgorithmLB,
    passwordMaster.value,
      (usingURL.value == NONE_CLICK_ACCOUNT_SETTINGS ? "" : usingURL.value)+ 
      cells[11].getAttribute("label") + // usernameTB
      cells[12].getAttribute("label"), // counter
    cells[2].getAttribute("label"), // whereLeetLB
    cells[3].getAttribute("label"), // leetLevelLB
    cells[5].getAttribute("label"), // passwordLength
    cells[13].getAttribute("label"), // charset
    cells[14].getAttribute("label"), // prefix
    cells[15].getAttribute("label")); // suffix
}

function importSettings() {
  copyFile(false);
  passwordMaker.refresh();
  init();
}

function exportSettings() {
  if (passwordMaker.ask(passwordMaker.findWindow(), "Export as HTML or RDF? Only RDF can be imported by PasswordMaker.", "HTML", "RDF")) {
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    fp.init(passwordMaker.findWindow(), "Export Settings to HTML File", nsIFilePicker.modeSave);
    //fp.setAttribute("windowtype", "passwordmaker");
    fp.defaultExtension = "html";
    fp.appendFilters(nsIFilePicker.filterAll);
    if (fp.show() == nsIFilePicker.returnCancel)
      return;
    if (fp.file.exists())
      fp.file.remove(false);
    isExporting = true; // defined in print.js
    PWDMKRFileIO.write(fp.file, rdfToHTML());
    passwordMaker.prompts.alert(window, "PasswordMaker",
      "Settings have been exported to " + fp.file.path);    
  }
  else {
    if (PwdMkr_RDFUtils.datasource.hasArcOut(passwordMaker.globalSettingsSubjectRes, passwordMaker.masterPasswordPredicateRes)) {
      if (!passwordMaker.ask(this,
        "Since you've saved the master password to disk, it will also be exported. Although the master password is encrypted, it is safer not to export at this time. It is recommended that you first remove the master password from disk by clearing the master password field or selecting one of the other master password storage options. Do you want to continue anyway?"))
        return;
    }
    copyFile(true);
  }
}

function copyFile(isExport) {
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
  fp.init(window, isExport?"Export Settings to RDF File":"Import Settings from File", isExport?nsIFilePicker.modeSave:nsIFilePicker.modeOpen);
  fp.defaultExtension = "rdf";
  fp.appendFilters(nsIFilePicker.filterAll);
  var res = fp.show();
  if (res == nsIFilePicker.returnCancel)
    return;

  var f1 = Components.classes["@mozilla.org/file/local;1"]
    .createInstance(Components.interfaces.nsILocalFile);

  var f2 = Components.classes["@mozilla.org/file/local;1"]
    .createInstance(Components.interfaces.nsILocalFile);

  if (!fp.file || !f1 || !f2)
    passwordMaker.prompts.alert(this, "PasswordMaker", "An error occured.");
  
  var pwmkrRDF = PWDMKRDirIO.get("ProfD"); // %Profile% dir
  pwmkrRDF.append("passwordmaker.rdf");

  try {
    f1.initWithFile(pwmkrRDF);
    if (isExport) {
      if (fp.file.exists())
        fp.file.remove(false);
      f2.initWithPath(fp.file.path.substring(0, fp.file.path.indexOf(fp.file.leafName)));
      f1.copyTo(f2, fp.file.leafName);
    }
    else {
      if (fp.file.path.substr(fp.file.path.length -4) != ".rdf") {
        passwordMaker.prompts.alert(this, "PasswordMaker", "PasswordMaker settings files must have .rdf extensions. Please rename the file and try again. Import cancelled.");
        return false;
      }
      if (!validateImportFile(fp.file)) {
        passwordMaker.prompts.alert(this, "PasswordMaker", fp.file.path + " is not a valid PasswordMaker file. Import canceled.");
        return false;
      }

      if (!passwordMaker.ask(this,
          "All existing settings (accounts, account settings, global settings, special domains, etc.) will be overwritten. ARE YOU SURE YOU WANT TO IMPORT THIS FILE?"))
          return false;

      if (pwmkrRDF.exists())
        pwmkrRDF.remove(false);
      f2.initWithPath(fp.file.path.substring(0, fp.file.path.indexOf(fp.file.leafName)));
      fp.file.copyTo(PWDMKRDirIO.get("ProfD"), "passwordmaker.rdf");
    }
    passwordMaker.prompts.alert(this, "PasswordMaker",
      isExport ? 
      ("Settings have been exported to " + fp.file.path + ". The file can be imported on other computers using PasswordMaker's File->Import option") :
      ("Settings have been imported from " + fp.file.path + ".")); 
    return true;
  }
  catch (e) {
    dump(e);
    passwordMaker.prompts.alert(this, "PasswordMaker", "An error occured and " + f1.path +
      " could not be copied to " + fp.file.path + ". Do you have write access to that directory?");
    return false;
  }
}

function validateImportFile(file) {
  try {
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var datasourceURI = ioService.newFileURI(file).spec;
    var datasource = PwdMkr_RDFUtils.rdfService.GetDataSourceBlocking(datasourceURI);
    var accountsSubjectRes = PwdMkr_RDFUtils.makeResource("http://passwordmaker.mozdev.org/accounts");
    return !PwdMkr_RDFUtils.rdfContainerUtils.IsEmpty(datasource, accountsSubjectRes);
    //TODO: even more validation
  }
  catch (e) {
    dump(e);
    return false;
  }
}

/**
 * Open the proper options dialog and close this one.
 */
function onOptionsDialogButton(whichOptionsDialog) {
  if (window.opener && passwordMaker)
    passwordMaker.switchOptionsDialog();  
  window.close();
}

function updateURLLabel() {
  // Update the "Using URL" label
  var url = getUrlToUse();
  if (!url || url == "")
    url = NONE_CLICK_ACCOUNT_SETTINGS;
  usingURL.value = url;
}

function sizeBasicOptions() {
  setTimeout(
  	function() {
  		var x = window.screenX, y = window.screenY;
  		window.moveTo(-2000, -2000); // move off-screen to prevent flashing
  		window.sizeToContent();
		var lve = document.getElementById("lastVisibleElement");
		window.resizeTo(window.outerWidth-90, lve.boxObject.y+65);
		window.moveTo(x, y);
  	}, 10);
}

function refreshMPW(quiet) {
  if (passwordMaker.mpw) {
    // User entered a master password when promtped from the context-menu or a pageLoad,
    // or it was stored as a preference and read at startup.
    passwordMaster.value = passwordMaster2.value = PwdMkr_MPW.getDecryptedMPW(passwordMaker.mpw);
  }
  else
    passwordMaster.value = passwordMaster2.value = "";
    
  passwordStorageLB.value = previousPasswordStorageLBValue = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.masterPasswordStoragePredicateRes); 
  _update(quiet);
  generatepassword();  
}